/**
**************************************************************************
* @file tester_config.h
* @brief Конфигурации для тестера диодов
**************************************************************************
* @defgroup    TESTER_CONFIGS    Configs for project
* @brief       Конфигурации для тестера диодов
* @details     
@{
*************************************************************************/
#ifndef _TESTER_CONFIG_H_
#define _TESTER_CONFIG_H_


#define TESTER_PULES_EXPETCED_WIDTH   4   ///< Предполагаемая длительность пика в отчетах ацп

/** 
	* @addtogroup TESTER_SW_TIMINGS_CONFIG   Configs for switching timings
	* @ingroup 		TESTER_CONFIGS
  * @brief      Конфигурации таймингов для ключей питания
	@{
	*/

/**
  * @brief    Задержка (миллисекундная) для положительного напряжения ПО УМОЛЧАНИЮ
  * @details  Пока только миллисекунды, т.к. меньше я пока не реализовал, да и как понимаю не требуется
  */
#define DEF_MS_TIME_FOR_POSITIVE              1000

/**
  * @brief    Задержка перед началом тестирования ПО УМОЛЧАНИЮ
  * @details  Задержка миллисекундная или тики for() @ref TIME_BEFORE_TEST_MS_DELAY
  */
#define DEF_TIME_BEFORE_TEST                  500
#define DEF_TIME_BEFORE_TEST_MS_DELAY         1         ///< включение миллисекундной задержки для @ref TIME_BEFORE_TEST по умолчанию

/**
  * @brief    Задержка между включением АЦП и предполагаемым скачком напряжения ПО УМОЛЧАНИЮ
  * @details  Задержка нужна, чтобы точно поймать его в буфере АЦП. 
  *
  *           Задержка миллисекундная или тики for() @ref TIME_BEFORE_TEST_MS_DELAY  
  * @note     Но если миллисекундная, то скорее всего скачок не попадет в буфер АЦП.
  *           Поэтому желательно только тики for() (@ref TIME_BEFORE_PEAK_MS_DELAY = 0)
  */
#define DEF_TIME_BEFORE_PEAK                  5
#define DEF_TIME_BEFORE_PEAK_MS_DELAY         0         ///< включение миллисекундной задержки для @ref TIME_BEFORE_PEAK(0 - задержка for(), 1 - миллисекундная)

/**
  * @brief    Задержка перед окончанием тестирования (отключение питания) ПО УМОЛЧАНИЮ
  * @details  Задержка миллисекундная или тики for() @ref TIME_BEFORE_DISCONNECT_MS_DELAY
  */
#define DEF_TIME_BEFORE_DISCONNECT            50
#define DEF_TIME_BEFORE_DISCONNECT_MS_DELAY   1         ///< включение миллисекундной задержки для @ref TIME_BEFORE_DISCONNECT (0 - задержка for(), 1 - миллисекундная)

/** TESTER_SW_TIMINGS_CONFIG
  * @}
  */

/** 
	* @addtogroup TESTER_POWER_SW_CONFIG   Configs for switches for power
	* @ingroup 		TESTER_CONFIGS
  * @brief      Конфигурации для ключей питания
	@{
	*/
//#define USE_HAL_GPIO_FUNCTIONS  ///< Использовать для переключения пинов HAL функции
//#define  ALL_SW_USE_SAME_PORT ///< Дефайн указывающий что все пины будут иметь один порт (для ускорения переключения)
  
/* Состояния ключей для подключения и откючения питания */
#define POWER_CONNECT       1   ///< Питание подключено в данном состоянии пина
#define POWER_DISCONNECT    0   ///< Питание отключено в данном состоянии пина

/* Ключи для подключения положительного питания к диоду */
#define SWITCH_DC_POSITIVE_Port       GPIOB         ///< Порт пина первого ключа для земли положительного источника питания
#define SWITCH_DC_POSITIVE_GND_Pin    GPIO_PIN_10   ///< Пин первого ключа для земли положительного источника питания
#define SWITCH_DC_POSITIVE_VDD_Pin    GPIO_PIN_11   ///< Пин второго ключа для напряжения положительного источника ппитания


/* Ключи для подключения отрицательного питания к диоду */
#define SWITCH_DC_NEGATIVE_Port       GPIOB         ///< Порт пина первого ключа для земли отрицательного источника ппитания
#define SWITCH_DC_NEGATIVE_GND_Pin    GPIO_PIN_0    ///< Пин первого ключа для земли отрицательного источника ппитания
#define SWITCH_DC_NEGATIVE_VDD_Pin    GPIO_PIN_1    ///< Пин первого ключа для напряжения отрицательного источника ппитания

/** TESTER_POWER_SW_CONFIG
  * @}
  */

/** 
	* @addtogroup TESTER_ADC_CONFIG   Configs for ADC
	* @ingroup 		TESTER_CONFIGS
  * @brief      Конфигурации для АЦП
	@{
	*/
  
#define ADC_BUFF_SIZE                           (200)               ///< Размер буфера АЦП
#define ADC_DMA_BUFF_SIZE                       (ADC_BUFF_SIZE)     ///< Размер буфера ДМА (1 канал)

#define ADC_U_CALIBR                            ((float)3.0)        ///< Калибровочное напряжение
#define ADC_VALUE_CALIBR                        (0xFFF)             ///< Значение АЦП при калибровочном напряжении @ref ADC_U_CALIBR
#define ADC_VALUE_ZERO                          (0)                 ///< Значение АЦП при нулевом напряжении
  
#define ADC_READ_TIMEOUT_MS                     20                  ///< Таймаут на ожидание разрядки конденсатора

/** TESTER_ADC_CONFIG
  * @}
  */


/** TESTER_CONFIGS
  * @}
  */

#endif //_TESTER_CONFIG_H_

