#include "tester_func.h"

TESTER_TestHandleTypeDef hTestDiode;

/**s
  * @brief    Инициализация хендла тестера диодов
  */
void TESTER_HandleInit(TESTER_TestHandleTypeDef *htest)
{
  /* Настройка структуры АЦП */
  htest->adc = &tester_adc;  
  TESTER_ADC_StructInit(htest->adc);
  
  /* Настройка пинов для подключения отрицательного источника */
  htest->DCNegSw.SW_Port    = SWITCH_DC_NEGATIVE_Port;
  htest->DCNegSw.SwGND_Pin  = SWITCH_DC_NEGATIVE_GND_Pin;
  htest->DCNegSw.SwVDD_Pin  = SWITCH_DC_NEGATIVE_VDD_Pin;
  
  /* Настройка пинов для подключения положительного источника */
  htest->DCPosSw.SW_Port    = SWITCH_DC_POSITIVE_Port;
  htest->DCPosSw.SwGND_Pin  = SWITCH_DC_POSITIVE_GND_Pin;
  htest->DCPosSw.SwVDD_Pin  = SWITCH_DC_POSITIVE_VDD_Pin;
  
  
  /* Настройка таймингов по умолчанию для тестирования */
  htest->SwTimings.msticks_for_positive_dc = DEF_MS_TIME_FOR_POSITIVE;
  
  htest->SwTimings.ticks_before_disconnect.ticks = DEF_TIME_BEFORE_DISCONNECT;
  htest->SwTimings.ticks_before_disconnect.msdelay = DEF_TIME_BEFORE_DISCONNECT_MS_DELAY;
  
  htest->SwTimings.ticks_before_go_to_peak.ticks = DEF_TIME_BEFORE_PEAK;
  htest->SwTimings.ticks_before_go_to_peak.msdelay = DEF_TIME_BEFORE_PEAK_MS_DELAY;
  
  htest->SwTimings.ticks_before_test.ticks = DEF_TIME_BEFORE_TEST;
  htest->SwTimings.ticks_before_test.msdelay = DEF_TIME_BEFORE_TEST_MS_DELAY;
  
  
  htest->continuous_buff_size = 20;
}


/**
  * @brief    Тест диодов: подключение прямого напряжения
  */
void TESTER_TestDiode_PositivePower(TESTER_TestHandleTypeDef *htest)
{
  /* Задержка, перед началом работы */
  TESTER_Delay(&htest->SwTimings.ticks_before_test);
  
  /* Включение континиус АЦП */
  HAL_ADC_Start(htest->adc->hadc);  
  
  /* Подкючение питания к диоду */
  TESTER_Connect_Power(&htest->DCPosSw);
  
  
  /* Подключение питания на определенное время */
  uint32_t tickstart = HAL_GetTick();
  HAL_StatusTypeDef res;
  while(1)
  {
    /* Считывание АЦП пока таймаут истечет */
    res = ADC_ReadContinuous(htest->adc, htest->continuous_buff_size, tickstart, htest->SwTimings.msticks_for_positive_dc);
    
    if(res != HAL_OK)
      break;
  }
  
  /* Отключение питания от диода */
  TESTER_Disconnect_Power(&htest->DCPosSw);
  
  /* Выключение континиус АЦП */
  HAL_ADC_Stop(htest->adc->hadc);
  
}

/**
  * @brief    Тест диодов: подключение обратного напряжения
  */
void TESTER_TestDiode_NegativePower(TESTER_TestHandleTypeDef *htest)
{
  /* Задержка, перед началом работы */
  TESTER_Delay(&htest->SwTimings.ticks_before_test);
  
  /* Включение АЦП */
  ADC_DMA_StartRead(htest->adc);
  
  /* Задержка, перед предполагаемым скачком */
  TESTER_Delay(&htest->SwTimings.ticks_before_go_to_peak);
  
  /* Подкючение питания к диоду */
  TESTER_Connect_Power(&htest->DCNegSw);
  
  /* Подключение питания на определенное время */
  TESTER_Delay(&htest->SwTimings.ticks_before_disconnect);
  
  /* Отключение питания от диода */
  TESTER_Disconnect_Power(&htest->DCNegSw);
  
  /* Обработка DMA */
  ADC_DMA_ReadForPeak(htest->adc, ADC_READ_TIMEOUT_MS);
  
}


/**
  * @brief    Тест диодов: подключение прямого, а потом обратного напряжения
  */
void TESTER_TestDiode_PosNegPower(TESTER_TestHandleTypeDef *htest)
{ 
  /* Задержка, перед началом работы */
  TESTER_Delay(&htest->SwTimings.ticks_before_test);
  
  /* Подкючение питания к диоду */
  TESTER_Connect_Power(&htest->DCPosSw);
  
  /* Подкючение положительного питания к диоду */
  msDelay(htest->SwTimings.msticks_for_positive_dc);
  
  /* Включение АЦП */
  ADC_DMA_StartRead(htest->adc);
  
  /* Ожидается задержка, перед предполагаемым скачком */
  TESTER_Delay(&htest->SwTimings.ticks_before_go_to_peak);
  
  /* Отключение питания от диода */
  TESTER_Reconnect_Power(&htest->DCPosSw, &htest->DCNegSw);
  
  /* Подключение отрицательного питания на определенное время */
  TESTER_Delay(&htest->SwTimings.ticks_before_disconnect);
  
  /* Отключение питания от диода */
  TESTER_Disconnect_Power(&htest->DCNegSw);
  
  /* Обработка DMA */
  ADC_DMA_ReadForPeak(htest->adc, ADC_READ_TIMEOUT_MS);
  
}

/**
  * @brief    Подключить питание с помощью ключей (пинов) в TESTER_PowerSwitchTypeDef
  */
void TESTER_Connect_Power(TESTER_PowerSwitchTypeDef *DCSw)
{
#ifdef USE_HAL_GPIO_FUNCTIONS // in tester_config.h
  HAL_GPIO_WritePin(DCSw->SW_Port, DCSw->SwGND_Pin | DCSw->SwVDD_Pin, POWER_CONNECT);
#else // USE_HAL_GPIO_FUNCTIONS  
  #if (POWER_CONNECT == 1)
    DCSw->SW_Port->BSRR = DCSw->SwGND_Pin | DCSw->SwVDD_Pin;
  #else //POWER_CONNECT == 1
    DCSw->SW_Port->BSRR = (DCSw->SwGND_Pin | DCSw->SwVDD_Pin) << 16;
  #endif //POWER_CONNECT == 1
#endif //USE_HAL_GPIO_FUNCTIONS
}
/**
  * @brief    Отключить питание с помощью ключей (пинов) в TESTER_PowerSwitchTypeDef
  */
void TESTER_Disconnect_Power(TESTER_PowerSwitchTypeDef *DCSw)
{
#ifdef USE_HAL_GPIO_FUNCTIONS
  HAL_GPIO_WritePin(DCSw->SW_Port, DCSw->SwGND_Pin | DCSw->SwVDD_Pin, POWER_DISCONNECT);
#else // USE_HAL_GPIO_FUNCTIONS
  #if (POWER_DISCONNECT == 0)
    DCSw->SW_Port->BSRR = (DCSw->SwGND_Pin | DCSw->SwVDD_Pin) << 16;
  #else //POWER_CONNECT == 1
    DCSw->SW_Port->BSRR = DCSw->SwGND_Pin | DCSw->SwVDD_Pin;
  #endif //POWER_CONNECT == 1
#endif //USE_HAL_GPIO_FUNCTIONS
}

/**
* @brief    Переключить питание с помощью ключей (пинов) в TESTER_PowerSwitchTypeDef
  */
void TESTER_Reconnect_Power(TESTER_PowerSwitchTypeDef *DCPosSw, TESTER_PowerSwitchTypeDef *DCNegSw)
{
#ifdef USE_HAL_GPIO_FUNCTIONS
  HAL_GPIO_WritePin(DCPosSw->SW_Port, DCPosSw->SwGND_Pin | DCPosSw->SwVDD_Pin, POWER_DISCONNECT);
  HAL_GPIO_WritePin(DCNegSw->SW_Port, DCNegSw->SwGND_Pin | DCNegSw->SwVDD_Pin, POWER_CONNECT);
#else // USE_HAL_GPIO_FUNCTIONS
  #ifdef ALL_SW_USE_SAME_PORT
    #if (POWER_CONNECT == 1) && (POWER_DISCONNECT == 0)
      DCPosSw->SW_Port->BSRR = ((DCNegSw->SwGND_Pin | DCNegSw->SwVDD_Pin) | ((DCPosSw->SwGND_Pin | DCPosSw->SwVDD_Pin) << 16));
    #elif (POWER_CONNECT == 1) && (POWER_DISCONNECT == 0)
      DCPosSw->SW_Port->BSRR = (((DCNegSw->SwGND_Pin | DCNegSw->SwVDD_Pin) << 16)| (DCPosSw->SwGND_Pin | DCPosSw->SwVDD_Pin));
    #endif //POWER_CONNECT && POWER_DISCONNECT
  #else //ALL_SW_USE_SAME_PORT  
    #if (POWER_DISCONNECT == 0)
      DCPosSw->SW_Port->BSRR = (DCPosSw->SwGND_Pin | DCPosSw->SwVDD_Pin) << 16;
    #else //POWER_DISCONNECT == 1
      DCPosSw->SW_Port->BSRR = DCPosSw->SwGND_Pin | DCPosSw->SwVDD_Pin;
    #endif //POWER_DISCONNECT
  
    #if (POWER_CONNECT == 1)
      DCNegSw->SW_Port->BSRR = DCNegSw->SwGND_Pin | DCNegSw->SwVDD_Pin;
    #else //POWER_CONNECT == 0
      DCNegSw->SW_Port->BSRR = (DCNegSw->SwGND_Pin | DCNegSw->SwVDD_Pin) << 16;
    #endif //POWER_CONNECT
  #endif //ALL_SW_USE_SAME_PORT
#endif //USE_HAL_GPIO_FUNCTIONS
}
/**
  * @brief    Формирование задержки (в тиках или миллисекундная)
  */
void TESTER_Delay(TESTER_TicksDelayTypeDef *tickdelay)
{
  /* Задержка, перед подключением питания */
  if(tickdelay->msdelay == 0) // если миллисекундная задержка выключена
  {
    /* Задержка, в тиках  */
    for(int i = 0; i < tickdelay->ticks; i++);
  }
  else // если миллисекундная задержка включена
  {
    /* Задержка, в миллисекундах */
    msDelay(tickdelay->ticks);
  }
}